<?php
// Configuracin de CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
    header("HTTP/1.1 200 OK");
    exit();
}

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('../config.php');

$response = [
    'success' => 0,
    'error' => [],
    'data' => [],
    'debug' => []
];

if (!isset($_SERVER['HTTP_AUTHORIZATION'])) {
    header('Content-Type: application/json');
    header('HTTP/1.1 401 Unauthorized');
    $response['error'][] = 'Authorization header missing';
    echo json_encode($response);
    exit();
}

$auth_header = $_SERVER['HTTP_AUTHORIZATION'];
list($type, $credentials) = explode(' ', $auth_header, 2);

if (strtolower($type) != 'basic') {
    header('Content-Type: application/json');
    header('HTTP/1.1 401 Unauthorized');
    $response['error'][] = 'Invalid authorization type';
    echo json_encode($response);
    exit();
}

if ($credentials != EXPECTED_CREDENTIALS) {
    header('Content-Type: application/json');
    header('HTTP/1.1 401 Unauthorized');
    $response['error'][] = 'Invalid credentials';
    echo json_encode($response);
    exit();
}

try {
    $connection = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

    if ($connection->connect_error) {
        throw new Exception('Connection failed: ' . $connection->connect_error);
    }
    $response['debug'][] = 'Connected to database';

    $db_prefix = DB_PREFIX;

    $data = json_decode(file_get_contents("php://input"), true);

    if (json_last_error() !== JSON_ERROR_NONE) {
        throw new Exception('JSON decode error: ' . json_last_error_msg());
    }

    $order_id = isset($data['order_id']) ? intval($data['order_id']) : 0;
    $customer_id = isset($data['customer_id']) ? intval($data['customer_id']) : 0;
    $amount = isset($data['amount']) ? floatval($data['amount']) : 0.00;
    $method_id = isset($data['method_id']) ? intval($data['method_id']) : 0;
    $method = isset($data['method']) ? $data['method'] : '';
    $reference = isset($data['reference']) ? $data['reference'] : null;
    $cclastfour = isset($data['cclastfour']) ? $data['cclastfour'] : null;
    $ccexpiration = isset($data['ccexpiration']) ? $data['ccexpiration'] : null;
    $ccaprovcode = isset($data['ccaprovcode']) ? $data['ccaprovcode'] : null;

    if ($order_id <= 0 || $customer_id <= 0 || $amount <= 0 || $method_id <= 0 || empty($method)) {
        throw new Exception("Invalid input data: order_id=$order_id, customer_id=$customer_id, amount=$amount, method_id=$method_id, method=$method");
    }

    $response['debug'][] = "Input data - Order ID: $order_id, Customer ID: $customer_id, Amount: $amount, Method ID: $method_id, Method: $method";

    // Insertar el pago
    $stmt = $connection->prepare("
        INSERT INTO `" . $db_prefix . "order_payment` 
        (`order_id`, `customer_id`, `amount`, `method_id`, `method`, `reference`, `cclastfour`, `ccexpiration`, `ccaprovcode`, `date_added`) 
        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, NOW())
    ");

    $stmt->bind_param("iisssssss", $order_id, $customer_id, $amount, $method_id, $method, $reference, $cclastfour, $ccexpiration, $ccaprovcode);

    if (!$stmt->execute()) {
        throw new Exception('Failed to record payment: ' . $stmt->error);
    }

    $payment_id = $stmt->insert_id;

    // Calcular el total de los pagos para este customer_id y order_id
    $total_query = $connection->prepare("
        SELECT SUM(`amount`) as total_payments 
        FROM `" . $db_prefix . "order_payment` 
        WHERE `order_id` = ? AND `customer_id` = ?
    ");
    $total_query->bind_param("ii", $order_id, $customer_id);
    $total_query->execute();
    $total_result = $total_query->get_result();
    $total_row = $total_result->fetch_assoc();
    $total_payments = $total_row['total_payments'];

    $response['success'] = 1;
    $response['data'][] = [
        'payment_id' => $payment_id,
        'order_id' => $order_id,
        'customer_id' => $customer_id,
        'amount' => $amount,
        'method_id' => $method_id,
        'method' => $method,
        'reference' => $reference,
        'cclastfour' => $cclastfour,
        'ccexpiration' => $ccexpiration,
        'ccaprovcode' => $ccaprovcode,
        'message' => 'Payment recorded successfully. Total payments for this order and customer: ' . number_format($total_payments, 2)
    ];

    $response['debug'][] = 'Payment recorded successfully. Total payments: ' . number_format($total_payments, 2);

} catch (Exception $e) {
    $response['error'][] = $e->getMessage();
}

if (empty($response)) {
    $response = ['success' => 0, 'error' => ['Empty response']];
}

header('Content-Type: application/json');
$json_response = json_encode($response);

if (json_last_error() !== JSON_ERROR_NONE) {
    echo json_encode(['success' => 0, 'error' => ['JSON encoding error: ' . json_last_error_msg()]]);
    exit();
}

echo $json_response;

if (isset($connection) && $connection instanceof mysqli) {
    $connection->close();
}
?>
