<?php
// Configuración de CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejo de solicitudes OPTIONS (preflight)
if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
    header("HTTP/1.1 200 OK");
    exit();
}

// Habilitar la visualización de errores para depuración
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Cargar la configuración de OpenCart
require_once('../config.php');

$response = [
    'success' => 0,
    'error' => [],
    'data' => [],
    'debug' => []
];

// Verificar encabezado de autenticación
if (!isset($_SERVER['HTTP_AUTHORIZATION'])) {
    header('Content-Type: application/json');
    header('HTTP/1.1 401 Unauthorized');
    $response['error'][] = 'Authorization header missing';
    echo json_encode($response);
    exit();
}

$auth_header = $_SERVER['HTTP_AUTHORIZATION'];
list($type, $credentials) = explode(' ', $auth_header, 2);

if (strtolower($type) != 'basic') {
    header('Content-Type: application/json');
    header('HTTP/1.1 401 Unauthorized');
    $response['error'][] = 'Invalid authorization type';
    echo json_encode($response);
    exit();
}

if ($credentials != EXPECTED_CREDENTIALS) {
    header('Content-Type: application/json');
    header('HTTP/1.1 401 Unauthorized');
    $response['error'][] = 'Invalid credentials';
    echo json_encode($response);
    exit();
}

try {
    // Conectar a la base de datos
    $connection = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

    if ($connection->connect_error) {
        throw new Exception('Connection failed: ' . $connection->connect_error);
    }
    $response['debug'][] = 'Connected to database';

    // Iniciar transacción
    $connection->begin_transaction();

    // Obtener datos de la solicitud
    $data = json_decode(file_get_contents("php://input"), true);

    if (json_last_error() !== JSON_ERROR_NONE) {
        throw new Exception('JSON decode error: ' . json_last_error_msg());
    }

    $order_id = isset($data['order_id']) ? intval($data['order_id']) : 0;
    $product_id = isset($data['product_id']) ? intval($data['product_id']) : 0;
    $quantity = isset($data['quantity']) ? intval($data['quantity']) : 1;
    $options = isset($data['options']) ? $data['options'] : [];

    if ($order_id <= 0 || $product_id <= 0 || $quantity <= 0) {
        throw new Exception("Invalid input data: order_id=$order_id, product_id=$product_id, quantity=$quantity");
    }

    $response['debug'][] = "Input data - Order ID: $order_id, Product ID: $product_id, Quantity: $quantity, Options: " . json_encode($options);

    // Obtener el precio del producto
    $price_query = "SELECT price FROM ocow_product WHERE product_id = $product_id";
    $result_price = $connection->query($price_query);

    if (!$result_price) {
        throw new Exception('Query failed: ' . $connection->error);
    }

    if ($result_price->num_rows > 0) {
        $row_price = $result_price->fetch_assoc();
        $price = floatval($row_price['price']);
        $response['debug'][] = "Product price from DB: $price";
    } else {
        throw new Exception('No price found for product ID: ' . $product_id);
    }

    // Calcular el total
    $total = $price * $quantity;
    $response['debug'][] = "Total calculated: $total";

    // Verificar si el producto ya existe en la orden
    $product_exists_query = "SELECT order_product_id, quantity, total FROM ocow_order_product WHERE order_id = $order_id AND product_id = $product_id";
    $result_product_exists = $connection->query($product_exists_query);

    if (!$result_product_exists) {
        throw new Exception('Query failed: ' . $connection->error);
    }

    if ($result_product_exists->num_rows > 0) {
        // Si el producto ya existe, actualizar la cantidad y el total
        $row_product_exists = $result_product_exists->fetch_assoc();
        $new_quantity = $row_product_exists['quantity'] + $quantity;
        $new_total = $row_product_exists['total'] + $total;
        $order_product_id = $row_product_exists['order_product_id'];

        $update_product_query = "UPDATE ocow_order_product SET quantity = $new_quantity, total = $new_total WHERE order_product_id = $order_product_id";
        if (!$connection->query($update_product_query)) {
            throw new Exception('Update failed: ' . $connection->error);
        }
        $response['data'][] = [
            'order_product_id' => $order_product_id,
            'new_quantity' => $new_quantity,
            'new_total' => $new_total,
            'message' => 'Product quantity updated successfully'
        ];
        $response['debug'][] = "Product quantity updated: $new_quantity, new total: $new_total";
    } else {
        // Si el producto no existe, insertarlo
        $add_product_query = "
            INSERT INTO ocow_order_product (order_id, product_id, name, model, quantity, price, total, tax, reward)
            SELECT $order_id, p.product_id, pd.name, p.model, $quantity, $price, $total, 0, 0
            FROM ocow_product p
            LEFT JOIN ocow_product_description pd ON p.product_id = pd.product_id
            WHERE p.product_id = $product_id";

        if (!$connection->query($add_product_query)) {
            throw new Exception('Insert failed: ' . $connection->error);
        }

        $order_product_id = $connection->insert_id;
        $response['data'][] = [
            'order_product_id' => $order_product_id,
            'quantity' => $quantity,
            'total' => $total,
            'message' => 'Product added to order successfully'
        ];
        $response['debug'][] = "Order product ID: $order_product_id";
    }

    // Insertar opciones del producto (si las hay)
    if (!empty($options)) {
        $response['debug'][] = "Processing options: " . json_encode($options);
        
        foreach ($options as $product_option_id => $product_option_value_id) {
            if (is_array($product_option_value_id)) {
                // Procesar opciones tipo checkbox que tienen múltiples valores
                foreach ($product_option_value_id as $value_id) {
                    processProductOption($connection, $order_id, $order_product_id, $product_option_id, $value_id, $response);
                }
            } else {
                // Procesar opciones tipo select que tienen un solo valor
                processProductOption($connection, $order_id, $order_product_id, $product_option_id, $product_option_value_id, $response);
            }
        }

        $response['debug'][] = "All product options processed successfully";
    } else {
        $response['debug'][] = "No options provided.";
    }

    // Calcular el subtotal de la orden
    $subtotal_query = "SELECT SUM(total) AS subtotal FROM ocow_order_product WHERE order_id = $order_id";
    $result_subtotal = $connection->query($subtotal_query);

    if (!$result_subtotal) {
        throw new Exception('Subtotal query failed: ' . $connection->error);
    }

    $row_subtotal = $result_subtotal->fetch_assoc();
    $subtotal = floatval($row_subtotal['subtotal']);
    $response['debug'][] = "Subtotal calculated: $subtotal";

    // Calcular el impuesto
    $tax_rate = 0.08625; // Reemplaza esto con la tasa de impuesto correspondiente
    $tax = $subtotal * $tax_rate;
    $response['debug'][] = "Tax calculated: $tax";

    // Calcular el total
    $total_query = "SELECT value FROM ocow_order_total WHERE order_id = $order_id AND code = 'shipping'";
    $result_total = $connection->query($total_query);

    if (!$result_total) {
        throw new Exception('Total query failed: ' . $connection->error);
    }

    $row_shipping = $result_total->fetch_assoc();
    $shipping_total = isset($row_shipping['value']) ? floatval($row_shipping['value']) : 0.0;
    $response['debug'][] = "Shipping total from DB: $shipping_total";

    $total = $subtotal + $tax + $shipping_total;
    $response['debug'][] = "Total calculated: $total";

    // Actualizar los valores en la tabla ocow_order_total
    $update_subtotal_query = "
        UPDATE ocow_order_total 
        SET value = $subtotal 
        WHERE order_id = $order_id AND code = 'sub_total'";
    if (!$connection->query($update_subtotal_query)) {
        throw new Exception('Update subtotal failed: ' . $connection->error);
    }
    $response['debug'][] = "Subtotal updated successfully";

    $update_tax_query = "
        UPDATE ocow_order_total 
        SET value = $tax 
        WHERE order_id = $order_id AND code = 'tax'";
    if (!$connection->query($update_tax_query)) {
        throw new Exception('Update tax failed: ' . $connection->error);
    }
    $response['debug'][] = "Tax updated successfully";

    $update_total_query = "
        UPDATE ocow_order_total 
        SET value = $total 
        WHERE order_id = $order_id AND code = 'total'";
    if (!$connection->query($update_total_query)) {
        throw new Exception('Update total failed: ' . $connection->error);
    }
    $response['debug'][] = "Total updated successfully";

    // Confirmar la transacción
    $connection->commit();

    $response['success'] = 1;
    $response['data'][] = [
        'order_id' => $order_id,
        'product_id' => $product_id,
        'quantity' => $quantity,
        'price' => $price,
        'total' => $total,
        'message' => 'Order totals updated successfully'
    ];

    $response['debug'][] = 'Totals updated successfully';

} catch (Exception $e) {
    // Revertir transacción en caso de error
    if (isset($connection) && $connection instanceof mysqli) {
        $connection->rollback();
    }
    $response['error'][] = $e->getMessage();
}

// Verificar que la respuesta no sea nula antes de enviarla
if (empty($response)) {
    $response = ['success' => 0, 'error' => ['Empty response']];
}

// Codificar y enviar la respuesta JSON
header('Content-Type: application/json');
$json_response = json_encode($response);

if (json_last_error() !== JSON_ERROR_NONE) {
    echo json_encode(['success' => 0, 'error' => ['JSON encoding error: ' . json_last_error_msg()]]);
    exit();
}

echo $json_response;

// Cerrar la conexión a la base de datos
if (isset($connection) && $connection instanceof mysqli) {
    $connection->close();
}

// Función para procesar e insertar opciones de producto
function processProductOption($connection, $order_id, $order_product_id, $product_option_id, $product_option_value_id, &$response) {
    $response['debug'][] = "Option - product_option_id: $product_option_id, product_option_value_id: $product_option_value_id";

    // Verifica manualmente que los IDs existan en la base de datos
    $verify_option_query = "
        SELECT po.option_id, od.name AS option_name, ovd.name AS option_value
        FROM ocow_product_option po 
        JOIN ocow_option_description od ON po.option_id = od.option_id
        JOIN ocow_product_option_value pov ON po.product_option_id = pov.product_option_id
        JOIN ocow_option_value_description ovd ON pov.option_value_id = ovd.option_value_id
        WHERE po.product_option_id = $product_option_id 
        AND pov.product_option_value_id = $product_option_value_id";

    $result_verify_option = $connection->query($verify_option_query);

    if (!$result_verify_option) {
        throw new Exception('Verification query failed: ' . $connection->error);
    }

    if ($result_verify_option->num_rows > 0) {
        $row_option = $result_verify_option->fetch_assoc();
        $option_name = $row_option['option_name'];
        $option_value = $row_option['option_value'];

        $response['debug'][] = "Verified option - Name: $option_name, Value: $option_value";

        // Inserta los valores obtenidos
        $option_query = "
            INSERT INTO ocow_order_option (order_id, order_product_id, product_option_id, product_option_value_id, name, value, type) 
            VALUES ($order_id, $order_product_id, $product_option_id, $product_option_value_id, '$option_name', '$option_value', 'select')";

        if (!$connection->query($option_query)) {
            throw new Exception('Insert failed for option: ' . $connection->error);
        }

        $response['debug'][] = "Option inserted successfully for product_option_id: $product_option_id, product_option_value_id: $product_option_value_id";
    } else {
        throw new Exception("No option details found for product_option_id: $product_option_id and product_option_value_id: $product_option_value_id");
    }
}
