<?php
// Configuración de CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejo de solicitudes OPTIONS (preflight)
if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
    header("HTTP/1.1 200 OK");
    exit();
}

// Carga la configuración de OpenCart
require_once('../config.php');

// Verificar encabezado de autenticación
if (!isset($_SERVER['HTTP_AUTHORIZATION'])) {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Authorization header missing'], 'data' => null]);
    exit();
}

$auth_header = $_SERVER['HTTP_AUTHORIZATION'];
list($type, $credentials) = explode(' ', $auth_header, 2);

if (strtolower($type) != 'basic') {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Invalid authorization type'], 'data' => null]);
    exit();
}

// Verificar las credenciales
if (trim($credentials) !== EXPECTED_CREDENTIALS) {
    // Agregar registros para depuración
    error_log("Received credentials: " . $credentials);
    error_log("Expected credentials: " . EXPECTED_CREDENTIALS);
    
    // Añadir los valores exactos para comparación en la respuesta JSON para depuración
    echo json_encode([
        'success' => 0,
        'error' => ['Invalid credentials'],
        'data' => null,
        'received' => $credentials,
        'expected' => EXPECTED_CREDENTIALS
    ]);
    exit();
}

// Conexión a la base de datos
$connection = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

if ($connection->connect_error) {
    error_log("Connection failed: " . $connection->connect_error);
    die("Connection failed: " . $connection->connect_error);
}

// Verificar que la solicitud sea POST
if ($_SERVER['REQUEST_METHOD'] == 'POST') {
    $data = json_decode(file_get_contents('php://input'), true);

    // Verificar que los datos necesarios estén presentes
    if (!isset($data['session_id']) || !isset($data['new_customer_id'])) {
        error_log("Missing required parameters");
        header('HTTP/1.1 400 Bad Request');
        echo json_encode(['success' => 0, 'error' => ['Missing required parameters'], 'data' => null]);
        exit();
    }

    $session_id = $data['session_id'];
    $new_customer_id = intval($data['new_customer_id']);

    if ($new_customer_id <= 0) {
        error_log("Invalid parameters: session_id = $session_id, new_customer_id = $new_customer_id");
        header('HTTP/1.1 400 Bad Request');
        echo json_encode(['success' => 0, 'error' => ['Invalid parameters'], 'data' => null]);
        exit();
    }

    // Actualizar el carrito con el nuevo customer_id
    $update_query = "UPDATE `" . DB_PREFIX . "cart` SET `customer_id` = ? WHERE `session_id` = ?";
    $stmt = $connection->prepare($update_query);
    if ($stmt === false) {
        error_log("Failed to prepare statement: " . $connection->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to prepare statement'], 'data' => null]);
        exit();
    }
    $stmt->bind_param('is', $new_customer_id, $session_id);
    if (!$stmt->execute()) {
        error_log("Failed to execute statement: " . $stmt->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to execute statement'], 'data' => null]);
        $stmt->close();
        $connection->close();
        exit();
    }
    if ($stmt->affected_rows <= 0) {
        error_log("No rows affected. Update query: $update_query, Session ID: $session_id, New Customer ID: $new_customer_id");
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to update customer ID in cart'], 'data' => null]);
        $stmt->close();
        $connection->close();
        exit();
    }
    $stmt->close();

    error_log("Customer ID updated successfully in cart. Session ID: $session_id, New Customer ID: $new_customer_id");
    header('HTTP/1.1 200 OK');
    echo json_encode(['success' => 1, 'error' => [], 'data' => [
        'message' => 'Customer ID updated successfully in cart'
    ]]);

    $connection->close();
} else {
    error_log("Method not allowed: " . $_SERVER['REQUEST_METHOD']);
    header('HTTP/1.1 405 Method Not Allowed');
    echo json_encode(['success' => 0, 'error' => ['Method not allowed'], 'data' => null]);
}
?>
