<?php
// Configuración de CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejo de solicitudes OPTIONS (preflight)
if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
    header("HTTP/1.1 200 OK");
    exit();
}

// Carga la configuración de OpenCart
require_once('../config.php');

// Verificar encabezado de autenticación
if (!isset($_SERVER['HTTP_AUTHORIZATION'])) {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Authorization header missing'], 'data' => null]);
    exit();
}

$auth_header = $_SERVER['HTTP_AUTHORIZATION'];
list($type, $credentials) = explode(' ', $auth_header, 2);

if (strtolower($type) != 'basic') {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Invalid authorization type'], 'data' => null]);
    exit();
}

if ($credentials != EXPECTED_CREDENTIALS) {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Invalid credentials'], 'data' => null]);
    exit();
}

// Conexión a la base de datos
$connection = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

if ($connection->connect_error) {
    error_log("Connection failed: " . $connection->connect_error);
    die("Connection failed: " . $connection->connect_error);
}

// Manejo de solicitudes DELETE
if ($_SERVER['REQUEST_METHOD'] == 'DELETE') {
    parse_str(file_get_contents("php://input"), $data);
    error_log("Received data: " . print_r($data, true));

    if (!isset($data['order_id']) || !isset($data['order_product_id'])) {
        error_log("Missing required parameters");
        header('HTTP/1.1 400 Bad Request');
        echo json_encode(['success' => 0, 'error' => ['Missing required parameters'], 'data' => null]);
        exit();
    }

    $order_id = intval($data['order_id']);
    $order_product_id = intval($data['order_product_id']);

    if ($order_id <= 0 || $order_product_id <= 0) {
        error_log("Invalid parameters: order_id = $order_id, order_product_id = $order_product_id");
        header('HTTP/1.1 400 Bad Request');
        echo json_encode(['success' => 0, 'error' => ['Invalid parameters'], 'data' => null]);
        exit();
    }

    // Eliminar el producto de la orden
    $delete_query = "DELETE FROM `" . DB_PREFIX . "order_product` WHERE order_id = ? AND order_product_id = ?";
    $stmt = $connection->prepare($delete_query);
    if ($stmt === false) {
        error_log("Failed to prepare statement: " . $connection->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to prepare statement'], 'data' => null]);
        exit();
    }
    $stmt->bind_param('ii', $order_id, $order_product_id);
    if (!$stmt->execute()) {
        error_log("Failed to execute statement: " . $stmt->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to execute statement'], 'data' => null]);
        $stmt->close();
        $connection->close();
        exit();
    }

    if ($stmt->affected_rows <= 0) {
        error_log("No rows affected. Delete query: $delete_query, Order ID: $order_id, Order Product ID: $order_product_id");
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to delete product from order'], 'data' => null]);
        $stmt->close();
        $connection->close();
        exit();
    }
    $stmt->close();

    // Recalcular los totales de la orden
    $subtotal_query = "SELECT SUM(total) as subtotal FROM `" . DB_PREFIX . "order_product` WHERE order_id = ?";
    $stmt = $connection->prepare($subtotal_query);
    if ($stmt === false) {
        error_log("Failed to prepare statement for subtotal query: " . $connection->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to prepare statement for subtotal query'], 'data' => null]);
        exit();
    }
    $stmt->bind_param('i', $order_id);
    $stmt->execute();
    $stmt->bind_result($subtotal);
    $stmt->fetch();
    $stmt->close();
    error_log("Subtotal: $subtotal");

    // Calcular nuevamente el impuesto y total
    // Este código es similar al del API anterior, calculando impuestos y actualizando la tabla order_total

    // [Aquí iría el código para recalcular impuestos y totales, similar al ejemplo anterior]

    error_log("Order totals updated successfully after product removal. Subtotal: $subtotal, Total: $total");
    header('HTTP/1.1 200 OK');
    echo json_encode(['success' => 1, 'error' => [], 'data' => [
        'message' => 'Product removed and order updated successfully',
        'totals' => [
            ['title' => 'Sub-Total', 'text' => '$' . number_format($subtotal, 2), 'value' => $subtotal],
            // Aquí también se devolverían los demás totales recalculados
        ]
    ]]);

    $connection->close();
} else {
    error_log("Method not allowed: " . $_SERVER['REQUEST_METHOD']);
    header('HTTP/1.1 405 Method Not Allowed');
    echo json_encode(['success' => 0, 'error' => ['Method not allowed'], 'data' => null]);
}
?>
