<?php
// Configuración de CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: PUT, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejo de solicitudes OPTIONS (preflight)
if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
    header("HTTP/1.1 200 OK");
    exit();
}

// Carga la configuración de OpenCart
require_once('../config.php'); // Asegúrate de que esta ruta es correcta y apunta al archivo config.php

// Verificar encabezado de autenticación
if (!isset($_SERVER['HTTP_AUTHORIZATION'])) {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Authorization header missing'], 'data' => null]);
    exit();
}

$auth_header = $_SERVER['HTTP_AUTHORIZATION'];
list($type, $credentials) = explode(' ', $auth_header, 2);

if (strtolower($type) != 'basic') {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Invalid authorization type'], 'data' => null]);
    exit();
}

if ($credentials != EXPECTED_CREDENTIALS) { // Asegúrate de definir EXPECTED_CREDENTIALS en tu configuración
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Invalid credentials'], 'data' => null]);
    exit();
}

// Conexión a la base de datos
$connection = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

if ($connection->connect_error) {
    error_log("Connection failed: " . $connection->connect_error);
    die("Connection failed: " . $connection->connect_error);
}

// Manejo de solicitudes PUT
if ($_SERVER['REQUEST_METHOD'] == 'PUT') {
    // Capturar los datos de la solicitud PUT
    parse_str(file_get_contents("php://input"), $data);

    // Mensajes de depuración para verificar los parámetros recibidos
    error_log("Received data: " . print_r($data, true));

    // Asegurarse de que los parámetros necesarios están presentes
    if (!isset($data['order_id']) || !isset($data['discount'])) {
        error_log("Missing required parameters");
        header('HTTP/1.1 400 Bad Request');
        echo json_encode(['success' => 0, 'error' => ['Missing required parameters'], 'data' => null]);
        exit();
    }

    $order_id = intval($data['order_id']);
    $discount = floatval($data['discount']);

    // Verificar que los parámetros sean válidos
    if ($order_id <= 0 || $discount < 0) {
        error_log("Invalid parameters: order_id = $order_id, discount = $discount");
        header('HTTP/1.1 400 Bad Request');
        echo json_encode(['success' => 0, 'error' => ['Invalid parameters'], 'data' => null]);
        exit();
    }

    // Recalcular los totales de la orden
    $subtotal_query = "SELECT SUM(total) as subtotal FROM `" . DB_PREFIX . "order_product` WHERE order_id = ?";
    $stmt = $connection->prepare($subtotal_query);
    $stmt->bind_param('i', $order_id);
    $stmt->execute();
    $stmt->bind_result($subtotal);
    $stmt->fetch();
    $stmt->close();

    if ($subtotal === null) {
        error_log("Order not found: order_id = $order_id");
        header('HTTP/1.1 404 Not Found');
        echo json_encode(['success' => 0, 'error' => ['Order not found'], 'data' => null]);
        exit();
    }

    $shipping = 5.00; // Valor fijo para el ejemplo
    $tax_rate = 0.1; // 10% de impuesto para el ejemplo
    $tax = number_format($subtotal * $tax_rate, 2, '.', '');
    $total = number_format($subtotal + $shipping + $tax - $discount, 2, '.', '');

    error_log("Recalculated Subtotal: $subtotal, Tax: $tax, Discount: $discount, Total: $total");

    // Actualizar los totales en la base de datos
    $update_totals_query = "UPDATE `" . DB_PREFIX . "order_total` SET value = ? WHERE order_id = ? AND title = ?";

    // Actualizar Sub-Total
    $title = 'Sub-Total';
    $stmt = $connection->prepare($update_totals_query);
    $stmt->bind_param('dis', $subtotal, $order_id, $title);
    $stmt->execute();

    // Actualizar Flat Shipping Rate
    $title = 'Flat Shipping Rate';
    $stmt = $connection->prepare($update_totals_query);
    $stmt->bind_param('dis', $shipping, $order_id, $title);
    $stmt->execute();

    // Actualizar Tax
    $title = 'NY State Tax';
    $stmt = $connection->prepare($update_totals_query);
    $stmt->bind_param('dis', $tax, $order_id, $title);
    $stmt->execute();

    // Actualizar o Insertar Discount
    $discount_query = "SELECT order_total_id FROM `" . DB_PREFIX . "order_total` WHERE order_id = ? AND title = 'Discount'";
    $stmt = $connection->prepare($discount_query);
    $stmt->bind_param('i', $order_id);
    $stmt->execute();
    $stmt->store_result();

    if ($stmt->num_rows > 0) {
        $stmt->bind_result($order_total_id);
        $stmt->fetch();
        $stmt->close();
        $update_discount_query = "UPDATE `" . DB_PREFIX . "order_total` SET value = ? WHERE order_id = ? AND title = 'Discount'";
        $stmt = $connection->prepare($update_discount_query);
        $stmt->bind_param('di', $discount, $order_id);
        $stmt->execute();
    } else {
        $stmt->close();
        $insert_discount_query = "INSERT INTO `" . DB_PREFIX . "order_total` (order_id, title, value, sort_order) VALUES (?, 'Discount', ?, 4)";
        $stmt = $connection->prepare($insert_discount_query);
        $stmt->bind_param('id', $order_id, $discount);
        $stmt->execute();
    }

    // Actualizar Total
    $title = 'Total';
    $stmt = $connection->prepare($update_totals_query);
    $stmt->bind_param('dis', $total, $order_id, $title);
    $stmt->execute();

    if ($stmt->affected_rows > 0) {
        error_log("Order totals updated successfully. Subtotal: $subtotal, Shipping: $shipping, Tax: $tax, Discount: $discount, Total: $total");
        header('HTTP/1.1 200 OK');
        echo json_encode(['success' => 1, 'error' => [], 'data' => [
            'message' => 'Discount applied successfully',
            'totals' => [
                ['title' => 'Sub-Total', 'text' => '$' . number_format($subtotal, 2), 'value' => $subtotal],
                ['title' => 'Flat Shipping Rate', 'text' => '$' . number_format($shipping, 2), 'value' => $shipping],
                ['title' => 'NY State Tax', 'text' => '$' . number_format($tax, 2), 'value' => $tax],
                ['title' => 'Discount', 'text' => '$' . number_format($discount, 2), 'value' => $discount],
                ['title' => 'Total', 'text' => '$' . number_format($total, 2), 'value' => $total]
            ]
        ]]);
    } else {
        error_log("Failed to update order totals. Subtotal: $subtotal, Shipping: $shipping, Tax: $tax, Discount: $discount, Total: $total");
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to update order totals'], 'data' => null]);
    }

    $stmt->close();
    $connection->close();
} else {
    error_log("Method not allowed: " . $_SERVER['REQUEST_METHOD']);
    header('HTTP/1.1 405 Method Not Allowed');
    echo json_encode(['success' => 0, 'error' => ['Method not allowed'], 'data' => null]);
}
?>
