<?php
// Configuración de CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejo de solicitudes OPTIONS (preflight)
if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
    header("HTTP/1.1 200 OK");
    exit();
}

// Carga la configuración de OpenCart
require_once('../config.php');

// Verificar encabezado de autenticación
if (!isset($_SERVER['HTTP_AUTHORIZATION'])) {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Authorization header missing'], 'data' => null]);
    exit();
}

$auth_header = $_SERVER['HTTP_AUTHORIZATION'];
list($type, $credentials) = explode(' ', $auth_header, 2);

if (strtolower($type) != 'basic' || $credentials != EXPECTED_CREDENTIALS) {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Invalid authorization type or credentials'], 'data' => null]);
    exit();
}

// Conexión a la base de datos
$connection = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

if ($connection->connect_error) {
    error_log("Connection failed: " . $connection->connect_error);
    die("Connection failed: " . $connection->connect_error);
}

// Manejo de solicitudes DELETE
if ($_SERVER['REQUEST_METHOD'] == 'DELETE') {
    $order_id = isset($_GET['order_id']) ? intval($_GET['order_id']) : 0;
    $order_product_id = isset($_GET['order_product_id']) ? intval($_GET['order_product_id']) : 0;

    if ($order_id <= 0 || $order_product_id <= 0) {
        header('HTTP/1.1 400 Bad Request');
        echo json_encode(['success' => 0, 'error' => ['Missing or invalid required parameters'], 'data' => null]);
        exit();
    }

    // Iniciar una transacción
    $connection->begin_transaction();

    try {
        // Eliminar el producto de la orden
        $delete_query = "DELETE FROM `" . DB_PREFIX . "order_product` WHERE order_id = ? AND order_product_id = ?";
        $stmt = $connection->prepare($delete_query);
        $stmt->bind_param('ii', $order_id, $order_product_id);
        $stmt->execute();

        if ($stmt->affected_rows === 0) {
            throw new Exception("Failed to remove product from order");
        }

        // Recalcular el subtotal y la cantidad total de productos
        $subtotal_query = "SELECT SUM(total) as subtotal FROM `" . DB_PREFIX . "order_product` WHERE order_id = ?";
        $stmt = $connection->prepare($subtotal_query);
        $stmt->bind_param('i', $order_id);
        $stmt->execute();
        $stmt->bind_result($subtotal);
        $stmt->fetch();
        $stmt->close();

        // Si no hay más productos, establecer subtotal a 0
        if ($subtotal === null) {
            $subtotal = 0.00;
        }

        // Valores de ejemplo para el cálculo
        $shipping = 0.00; // En este caso, se supone que no hay costo de envío
        $tax_rate = 0.08625; // Impuesto del 8.625% para el ejemplo
        $tax = round($subtotal * $tax_rate, 2); // Asegúrate de redondear el impuesto a 2 decimales

        // Calcular el total correctamente
        $total_raw = round($subtotal + $tax + $shipping, 2); // Asegúrate de redondear el total a 2 decimales

        // Asegurarse de que los valores se calculan correctamente
        error_log("Subtotal: $subtotal, Shipping: $shipping, Tax: $tax, Total: $total_raw");

        // Actualizar los totales en la base de datos
        $update_totals_query = "
            UPDATE `" . DB_PREFIX . "order_total` 
            SET value = CASE 
                WHEN code = 'sub_total' THEN ?
                WHEN code = 'shipping' THEN ?
                WHEN code = 'tax' THEN ?
                WHEN code = 'total' THEN ?
                ELSE value
            END
            WHERE order_id = ? AND code IN ('sub_total', 'shipping', 'tax', 'total')";

        $stmt = $connection->prepare($update_totals_query);
        $stmt->bind_param('ddddi', $subtotal, $shipping, $tax, $total_raw, $order_id);
        $stmt->execute();

        if ($stmt->affected_rows === 0) {
            throw new Exception("Failed to update order totals");
        }

        // Confirmar la transacción
        $connection->commit();

        header('HTTP/1.1 200 OK');
        echo json_encode(['success' => 1, 'error' => [], 'data' => [
            'message' => 'Product removed and order updated successfully',
            'totals' => [
                ['code' => 'sub_total', 'title' => 'Sub-Total', 'value' => $subtotal],
                ['code' => 'shipping', 'title' => 'Flat Shipping Rate', 'value' => $shipping],
                ['code' => 'tax', 'title' => 'NY State Tax', 'value' => $tax],
                ['code' => 'total', 'title' => 'Total', 'value' => $total_raw]
            ]
        ]]);

    } catch (Exception $e) {
        // Deshacer la transacción en caso de error
        $connection->rollback();
        error_log($e->getMessage());
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => [$e->getMessage()], 'data' => null]);
    }

    $stmt->close();
    $connection->close();
} else {
    header('HTTP/1.1 405 Method Not Allowed');
    echo json_encode(['success' => 0, 'error' => ['Method not allowed'], 'data' => null]);
}
?>
