<?php
// Configuración de CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: PUT, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejo de solicitudes OPTIONS (preflight)
if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
    header("HTTP/1.1 200 OK");
    exit();
}

// Carga la configuración de OpenCart
require_once('../config.php');

// Verificar encabezado de autenticación
if (!isset($_SERVER['HTTP_AUTHORIZATION'])) {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Authorization header missing'], 'data' => null]);
    exit();
}

$auth_header = $_SERVER['HTTP_AUTHORIZATION'];
list($type, $credentials) = explode(' ', $auth_header, 2);

if (strtolower($type) != 'basic') {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Invalid authorization type'], 'data' => null]);
    exit();
}

if ($credentials != EXPECTED_CREDENTIALS) {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Invalid credentials'], 'data' => null]);
    exit();
}

// Conexión a la base de datos
$connection = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

if ($connection->connect_error) {
    error_log("Connection failed: " . $connection->connect_error);
    die("Connection failed: " . $connection->connect_error);
}

// Manejo de solicitudes PUT
if ($_SERVER['REQUEST_METHOD'] == 'PUT') {
    parse_str(file_get_contents("php://input"), $data);
    error_log("Received data: " . print_r($data, true));

    if (!isset($data['order_id']) || !isset($data['order_product_id']) || !isset($data['price']) || !isset($data['quantity'])) {
        error_log("Missing required parameters");
        header('HTTP/1.1 400 Bad Request');
        echo json_encode(['success' => 0, 'error' => ['Missing required parameters'], 'data' => null]);
        exit();
    }

    $order_id = intval($data['order_id']);
    $order_product_id = intval($data['order_product_id']);
    $price = floatval($data['price']);
    $quantity = intval($data['quantity']);

    if ($order_id <= 0 || $order_product_id <= 0 || $price <= 0 || $quantity <= 0) {
        error_log("Invalid parameters: order_id = $order_id, order_product_id = $order_product_id, price = $price, quantity = $quantity");
        header('HTTP/1.1 400 Bad Request');
        echo json_encode(['success' => 0, 'error' => ['Invalid parameters'], 'data' => null]);
        exit();
    }

    $total = number_format($price * $quantity, 2, '.', '');
    error_log("Calculated Total: $total, Price: $price, Quantity: $quantity");

    // Actualizar la cantidad, el precio y el total del producto en la orden
    $update_query = "UPDATE `" . DB_PREFIX . "order_product` SET quantity = ?, price = ?, total = ? WHERE order_id = ? AND order_product_id = ?";
    $stmt = $connection->prepare($update_query);
    if ($stmt === false) {
        error_log("Failed to prepare statement: " . $connection->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to prepare statement'], 'data' => null]);
        exit();
    }
    $stmt->bind_param('isdii', $quantity, $price, $total, $order_id, $order_product_id);
    if (!$stmt->execute()) {
        error_log("Failed to execute statement: " . $stmt->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to execute statement'], 'data' => null]);
        $stmt->close();
        $connection->close();
        exit();
    }
    if ($stmt->affected_rows <= 0) {
        error_log("No rows affected. Update query: $update_query, Quantity: $quantity, Price: $price, Total: $total, Order ID: $order_id, Order Product ID: $order_product_id");
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to update product quantity and price in order'], 'data' => null]);
        $stmt->close();
        $connection->close();
        exit();
    }
    $stmt->close();

    // Establecer el costo de envío a $0.00
    $shipping = 0.00;
    error_log("Shipping Cost: $shipping");

    // Recalcular los totales de la orden
    $subtotal_query = "SELECT SUM(total) as subtotal FROM `" . DB_PREFIX . "order_product` WHERE order_id = ?";
    $stmt = $connection->prepare($subtotal_query);
    if ($stmt === false) {
        error_log("Failed to prepare statement for subtotal query: " . $connection->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to prepare statement for subtotal query'], 'data' => null]);
        exit();
    }
    $stmt->bind_param('i', $order_id);
    $stmt->execute();
    $stmt->bind_result($subtotal);
    $stmt->fetch();
    $stmt->close();
    error_log("Subtotal: $subtotal");

    // Obtener los productos de la orden para calcular los impuestos
    $tax_query = "
        SELECT op.product_id, p.tax_class_id, tr.rate, op.total as product_total
        FROM `" . DB_PREFIX . "order_product` op
        INNER JOIN `" . DB_PREFIX . "product` p ON op.product_id = p.product_id
        INNER JOIN `" . DB_PREFIX . "tax_rule` trr ON p.tax_class_id = trr.tax_class_id
        INNER JOIN `" . DB_PREFIX . "tax_rate` tr ON trr.tax_rate_id = tr.tax_rate_id
        WHERE op.order_id = ?
    ";
    $stmt = $connection->prepare($tax_query);
    if ($stmt === false) {
        error_log("Failed to prepare statement for tax query: " . $connection->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to prepare statement for tax query'], 'data' => null]);
        exit();
    }
    $stmt->bind_param('i', $order_id);
    $stmt->execute();
    $stmt->bind_result($product_id, $tax_class_id, $tax_rate, $product_total);

    $total_tax = 0;
    while ($stmt->fetch()) {
        $tax_amount = $product_total * ($tax_rate / 100);
        $total_tax += $tax_amount;
        error_log("Product ID: $product_id, Tax Class ID: $tax_class_id, Tax Rate: $tax_rate, Product Total: $product_total, Tax Amount: $tax_amount");
    }
    $stmt->close();

    $total_tax = number_format($total_tax, 2, '.', '');
    $total = number_format($subtotal + $shipping + $total_tax, 2, '.', '');

    error_log("Recalculated Subtotal: $subtotal, Tax: $total_tax, Total: $total");

    // Actualizar los totales en la base de datos
    $update_totals_query = "UPDATE `" . DB_PREFIX . "order_total` SET value = ? WHERE order_id = ? AND title = ?";

    // Actualizar Sub-Total
    $title = 'Sub-Total';
    $stmt = $connection->prepare($update_totals_query);
    if ($stmt === false) {
        error_log("Failed to prepare statement for updating subtotal: " . $connection->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to prepare statement for updating subtotal'], 'data' => null]);
        exit();
    }
    $stmt->bind_param('dis', $subtotal, $order_id, $title);
    if (!$stmt->execute()) {
        error_log("Failed to execute statement for updating subtotal: " . $stmt->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to execute statement for updating subtotal'], 'data' => null]);
        $stmt->close();
        $connection->close();
        exit();
    }
    $stmt->close();

    // Actualizar Flat Shipping Rate
    $title = 'Flat Shipping Rate';
    $stmt = $connection->prepare($update_totals_query);
    if ($stmt === false) {
        error_log("Failed to prepare statement for updating shipping: " . $connection->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to prepare statement for updating shipping'], 'data' => null]);
        exit();
    }
    $stmt->bind_param('dis', $shipping, $order_id, $title);
    if (!$stmt->execute()) {
        error_log("Failed to execute statement for updating shipping: " . $stmt->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to execute statement for updating shipping'], 'data' => null]);
        $stmt->close();
        $connection->close();
        exit();
    }
    $stmt->close();

    // Actualizar Tax
    $title = 'NY State Tax';
    $stmt = $connection->prepare($update_totals_query);
    if ($stmt === false) {
        error_log("Failed to prepare statement for updating tax: " . $connection->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to prepare statement for updating tax'], 'data' => null]);
        exit();
    }
    $stmt->bind_param('dis', $total_tax, $order_id, $title);
    if (!$stmt->execute()) {
        error_log("Failed to execute statement for updating tax: " . $stmt->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to execute statement for updating tax'], 'data' => null]);
        $stmt->close();
        $connection->close();
        exit();
    }
    $stmt->close();

    // Actualizar Total
    $title = 'Total';
    $stmt = $connection->prepare($update_totals_query);
    if ($stmt === false) {
        error_log("Failed to prepare statement for updating total: " . $connection->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to prepare statement for updating total'], 'data' => null]);
        exit();
    }
    $stmt->bind_param('dis', $total, $order_id, $title);
    if (!$stmt->execute()) {
        error_log("Failed to execute statement for updating total: " . $stmt->error);
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode(['success' => 0, 'error' => ['Failed to execute statement for updating total'], 'data' => null]);
        $stmt->close();
        $connection->close();
        exit();
    }
    $stmt->close();

    error_log("Order totals updated successfully. Subtotal: $subtotal, Shipping: $shipping, Tax: $total_tax, Total: $total");
    header('HTTP/1.1 200 OK');
    echo json_encode(['success' => 1, 'error' => [], 'data' => [
        'message' => 'Order updated successfully',
        'totals' => [
            ['title' => 'Sub-Total', 'text' => '$' . number_format($subtotal, 2), 'value' => $subtotal],
            ['title' => 'Flat Shipping Rate', 'text' => '$' . number_format($shipping, 2), 'value' => $shipping],
            ['title' => 'NY State Tax', 'text' => '$' . number_format($total_tax, 2), 'value' => $total_tax],
            ['title' => 'Total', 'text' => '$' . number_format($total, 2), 'value' => $total]
        ]
    ]]);

    $connection->close();
} else {
    error_log("Method not allowed: " . $_SERVER['REQUEST_METHOD']);
    header('HTTP/1.1 405 Method Not Allowed');
    echo json_encode(['success' => 0, 'error' => ['Method not allowed'], 'data' => null]);
}
?>
